import bpy
import sys
import imp
import time
import numpy as np
from os import system

np.set_printoptions(threshold=np.nan)

print('starting simulation')

import Geometry
imp.reload(Geometry)
import Computation
imp.reload(Computation)

# Generate mesh data
wing = Geometry.MeshData('wing')
wing.findStrips(2*30-1)
wake = Geometry.MeshData('wake')
wake.findStrips(2*30-1)

wingList = [wing]
wakeList = [wake]

# Set up OpenCL
opencl = Computation.Computation(workPath)
wing.setOpenCL(opencl.cntxt)
wake.setOpenCL(opencl.cntxt)

# External velocity vector
U = np.array([1.0, 0, 0])

# System to be solved
for i in range(3):
    print('Iteration:', i)
    B = opencl.influenceMatrixSourceDirichlet(wingList)
    A = opencl.influenceMatrixDoubletDirichlet(wingList)
    opencl.influenceFromStripsDirichlet(A, wingList, wakeList)

    sigma = opencl.rightSideSigma(wingList, U)
    b = -np.dot(B, sigma)

    gamma = np.linalg.solve(A, b)

    # Set strengths
    wing.setGamma(gamma, opencl.cntxt)
    wing.setSigma(sigma, opencl.cntxt)
    wake.setGammaWake(wing, opencl.cntxt)

    opencl.deformWake(wake, wingList, wakeList, U)

    wing.postProcessingDirichlet(U)

    print('Pressure Force:', wing.Force[0], wing.Force[1], wing.Force[2])

    F = opencl.forceCalculation(wake, wing, wakeList, wingList, U)

    print('Wing Kutta-joukowski Force:', F[0], F[1], F[2])
    
    wake.updateData(opencl.cntxt)
    wing.Force = np.zeros(3)

wing.velocityColorMap()
wing.pressureColorMap()

print('ending simulation')
print('\n')

