// Prototypes
#define error 0.000000001

float4 multiplyMatrix(float4 p, float4 X, float4 Y, float4 Z);

float4 sourceLineVelocity(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n);
float4 doubletLineVelocity(float4 p, float4 p1, float4 p2);

float sourceLinePotential(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n);
float doubletLinePotential(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n);

float calcC(float PN, float RNUM, float DNOM);

float4 unitSourceVelocity(int i,
						  float4 p,
						  __global float4 *vertices,
						  __global float4 *ctrlPoints,
						  __global float4 *l,
						  __global float4 *m,
						  __global float4 *n,
						  __global int4   *topology);

float4 unitDoubletVelocity(int i,
						   float4 p,
						   __global float4 *vertices,
						   __global float4 *ctrlPoints,
						   __global float4 *l,
						   __global float4 *m,
						   __global float4 *n,
						   __global int4   *topology);

float unitSourcePotential(int i,
						  float4 p,
						  __global float4 *vertices,
						  __global float4 *ctrlPoints,
						  __global float4 *l,
						  __global float4 *m,
						  __global float4 *n,
						  __global int4   *topology);

float unitDoubletPotential(int i,
						   float4 p,
						   __global float4 *vertices,
						   __global float4 *ctrlPoints,
						   __global float4 *l,
						   __global float4 *m,
						   __global float4 *n,
						   __global int4   *topology);

// Implementation

float4 multiplyMatrix(float4 p, float4 X, float4 Y, float4 Z) {
	float4 pOut;

	pOut.x = p.x*X.x + p.y*X.y + p.z*X.z;
	pOut.y = p.x*Y.x + p.y*Y.y + p.z*Y.z;
	pOut.z = p.x*Z.x + p.y*Z.y + p.z*Z.z;

	return pOut;
}

float calcC(float PN, float RNUM, float DNOM) {
	float C;

	if (PN == 0 && DNOM < 0) {
		C = -M_PI;
	}
	else if (PN == 0 && DNOM == 0) {
		C = -0.5*M_PI;
	}
	else if (PN == 0 && DNOM > 0) {
		C = 0.0;
	}
	else {
		C = atan2(RNUM, DNOM);
	}

	return C;
}

float4 sourceLineVelocity(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n) {
	float4 u; // Return variable

	float4 s = p2 - p1;
	float4 a = p  - p1;
	float4 b = p  - p2;

	float S = sqrt(s.x*s.x + s.y*s.y + s.z*s.z);

	if (S > error) {
		float A = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);
		float B = sqrt(b.x*b.x + b.y*b.y + b.z*b.z);

		float PN = p.x*n.x + p.y*n.y + p.z*n.z;

		float SL = s.x*l.x + s.y*l.y + s.z*l.z;
		float SM = s.x*m.x + s.y*m.y + s.z*m.z;

		float AL = a.x*l.x + a.y*l.y + a.z*l.z;
		float AM = a.x*m.x + a.y*m.y + a.z*m.z;

		float Al = AM*SL - AL*SM;
		
		float PA = PN*PN*SL + Al*AM;
		float PB = PA - Al*SM;

		float RNUM = SM*PN*(B*PA - A*PB);
		float DNOM = PA*PB + PN*PN*A*B*SM*SM;
	 
		float GL = (1/S)*log( fabs((A+B+S)/(A+B-S)) );

		float C = calcC(PN, RNUM, DNOM);

		u = GL*SM*l - GL*SL*m + C*n;
	}
	else {
		u.x = 0;
		u.y = 0;
		u.z = 0;
	}

	return u;
}

float4 doubletLineVelocity(float4 p, float4 p1, float4 p2) {
	float4 u; // Return variable

	float4 a = p - p1;
	float4 b = p - p2;
	float4 s = p2 - p1;

	float AVBx = a.y*b.z - a.z*b.y;
	float AVBy = a.z*b.x - a.x*b.z;
	float AVBz = a.x*b.y - a.y*b.x;

	float AVB = AVBx*AVBx + AVBy*AVBy + AVBz*AVBz;

	float A = sqrt( a.x*a.x + a.y*a.y + a.z*a.z );
	float B = sqrt( b.x*b.x + b.y*b.y + b.z*b.z );
	float S = sqrt( s.x*s.x + s.y*s.y + s.z*s.z );

	if (A < error || B < error || AVB < error) {
		u.x = 0;
		u.y = 0;
		u.z = 0;
	}
	else if (S < error) {
		u.x = 0;
		u.y = 0;
		u.z = 0;
	}
	else {
		float ADB = a.x*b.x + a.y*b.y + a.z*b.z;

		float K = (A+B)/( A*B*(A*B + ADB) );

		u.x = K*AVBx;
		u.y = K*AVBy; 
		u.z = K*AVBz;
	}

	return u;
}

float sourceLinePotential(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n) {
	float4 s = p2 - p1;
	float4 a = p  - p1;
	float4 b = p  - p2;

	float S = sqrt(s.x*s.x + s.y*s.y + s.z*s.z);

	float phi;

	if (S > error) {
		float A = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);
		float B = sqrt(b.x*b.x + b.y*b.y + b.z*b.z);

		float PN = p.x*n.x + p.y*n.y + p.z*n.z;

		float SL = s.x*l.x + s.y*l.y + s.z*l.z;
		float SM = s.x*m.x + s.y*m.y + s.z*m.z;

		float AL = a.x*l.x + a.y*l.y + a.z*l.z;
		float AM = a.x*m.x + a.y*m.y + a.z*m.z;
		
		float Al = AM*SL - AL*SM;

		float PA = PN*PN*SL + Al*AM;
		float PB = PA - Al*SM;

		float RNUM = SM*PN*(B*PA - A*PB);
		float DNOM = PA*PB + PN*PN*A*B*SM*SM;

		float C = calcC(PN, RNUM, DNOM); 
	 
		float GL = (1/S)*log( fabs( (A+B+S)/(A+B-S) ) );

		phi = Al*GL - PN*C;
	}
	else {
		phi = 0;
	}

	return phi;
}

float doubletLinePotential(float4 p, float4 p1, float4 p2, float4 l, float4 m, float4 n) {
	float4 s = p2 - p1;
	float4 a = p  - p1;
	float4 b = p  - p2;

	float S = sqrt(s.x*s.x + s.y*s.y + s.z*s.z);

	float phi;

	if (S > error) {

		float A = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);
		float B = sqrt(b.x*b.x + b.y*b.y + b.z*b.z);

		float PN = p.x*n.x + p.y*n.y + p.z*n.z;

		float SL = s.x*l.x + s.y*l.y + s.z*l.z;
		float SM = s.x*m.x + s.y*m.y + s.z*m.z;

		float AL = a.x*l.x + a.y*l.y + a.z*l.z;
		float AM = a.x*m.x + a.y*m.y + a.z*m.z;

		float Al = AM*SL - AL*SM;

		float PA = PN*PN*SL + Al*AM;
		float PB = PA - Al*SM;

		float RNUM = SM*PN*(B*PA - A*PB);
		float DNOM = PA*PB + PN*PN*A*B*SM*SM;

		phi = calcC(PN, RNUM, DNOM); 
		
	}
	else {
		phi = 0;
	}

	return phi;
}

float4 unitSourceVelocity(int i,
						  float4 p,
						  __global float4 *vertices,
						  __global float4 *ctrlPoints,
						  __global float4 *l,
						  __global float4 *m,
						  __global float4 *n,
						  __global int4   *topology) {
	float4 u;

	float4 p0 = ctrlPoints[i]; //Center of panel

	p = p - p0; // Translate point into new panel centerd coordinate system

	// Corner points
	int i1 = topology[i][0]; 
	int i2 = topology[i][1];
	int i3 = topology[i][2];
	int i4 = topology[i][3];

	float4 p1 = vertices[i1] - p0;
	float4 p2 = vertices[i2] - p0;
	float4 p3 = vertices[i3] - p0;
	float4 p4 = vertices[i4] - p0;

	// Find transformation matrices between global and local panel coordinate system
	float4 X = l[i];
	float4 Y = m[i];
	float4 Z = n[i];

	float PL = p.x*X.x + p.y*X.y + p.z*X.z;
	float PM = p.x*Y.x + p.y*Y.y + p.z*Y.z;
	float PN = p.x*Z.x + p.y*Z.y + p.z*Z.z;

	float4 u1 = sourceLineVelocity(p, p1, p2, l[i], m[i], n[i]);
	float4 u2 = sourceLineVelocity(p, p2, p3, l[i], m[i], n[i]);
	float4 u3 = sourceLineVelocity(p, p3, p4, l[i], m[i], n[i]);
	float4 u4 = sourceLineVelocity(p, p4, p1, l[i], m[i], n[i]);

	u = u1 + u2 + u3 + u4;

	if (fabs(PN) < error) {
		if ( fabs(PL) < error && fabs(PM) < error ) {
			u.z = 2*M_PI;
			u.x = 0;
			u.y = 0;
		}
		else {
			u.z = 0;
		}
	}

	return u;
}

float4 unitDoubletVelocity(int i,
						   float4 p,
						   __global float4 *vertices,
						   __global float4 *ctrlPoints,
						   __global float4 *l,
						   __global float4 *m,
						   __global float4 *n,
						   __global int4   *topology) {

	float4 p0 = ctrlPoints[i]; //Center of panel

	p = p - p0; // Translate point into new panel centerd coordinate system

	// Corner points
	int i1 = topology[i][0]; 
	int i2 = topology[i][1];
	int i3 = topology[i][2];
	int i4 = topology[i][3];

	float4 p1 = vertices[i1] - p0;
	float4 p2 = vertices[i2] - p0;
	float4 p3 = vertices[i3] - p0;
	float4 p4 = vertices[i4] - p0;

	// Calculate contribution from every line
	float4 u1 = doubletLineVelocity(p, p1, p2);
	float4 u2 = doubletLineVelocity(p, p2, p3);
	float4 u3 = doubletLineVelocity(p, p3, p4);
	float4 u4 = doubletLineVelocity(p, p4, p1);

	float4 u = u1 + u2 + u3 + u4;

	return u;
}

float unitSourcePotential(int i,
						  float4 p,
						  __global float4 *vertices,
						  __global float4 *ctrlPoints,
						  __global float4 *l,
						  __global float4 *m,
						  __global float4 *n,
						  __global int4   *topology) {

	float4 p0 = ctrlPoints[i]; //Center of panel

	p = p - p0; // Translate point into new panel centerd coordinate system

	// Corner points
	int i1 = topology[i][0]; 
	int i2 = topology[i][1];
	int i3 = topology[i][2];
	int i4 = topology[i][3];

	float4 p1 = vertices[i1] - p0;
	float4 p2 = vertices[i2] - p0;
	float4 p3 = vertices[i3] - p0;
	float4 p4 = vertices[i4] - p0;

	float phi1 = sourceLinePotential(p, p1, p2, l[i], m[i], n[i]);
	float phi2 = sourceLinePotential(p, p2, p3, l[i], m[i], n[i]);
	float phi3 = sourceLinePotential(p, p3, p4, l[i], m[i], n[i]);
	float phi4 = sourceLinePotential(p, p4, p1, l[i], m[i], n[i]);

	return phi1 + phi2 + phi3 + phi4; 
}

float unitDoubletPotential(int i,
						   float4 p,
						   __global float4 *vertices,
						   __global float4 *ctrlPoints,
						   __global float4 *l,
						   __global float4 *m,
						   __global float4 *n,
						   __global int4   *topology) {

	float4 p0 = ctrlPoints[i]; //Center of panel

	float phi;

	p = p - p0; // Translate point into new panel centerd coordinate system

	// Corner points
	int i1 = topology[i][0]; 
	int i2 = topology[i][1];
	int i3 = topology[i][2];
	int i4 = topology[i][3];

	float4 p1 = vertices[i1] - p0;
	float4 p2 = vertices[i2] - p0;
	float4 p3 = vertices[i3] - p0;
	float4 p4 = vertices[i4] - p0;

	// Find transformation matrices between global and local panel coordinate system
	float4 X = l[i];
	float4 Y = m[i];
	float4 Z = n[i];

	float phi1 = doubletLinePotential(p, p1, p2, X, Y, Z);
	float phi2 = doubletLinePotential(p, p2, p3, X, Y, Z);
	float phi3 = doubletLinePotential(p, p3, p4, X, Y, Z);
	float phi4 = doubletLinePotential(p, p4, p1, X, Y, Z);

	phi = phi1 + phi2 + phi3 + phi4;


	return phi;
}

__kernel void sourceInfluenceNewmann(float4 p,
							  		 float4 n_p,
							  		 __global float4 *vertices,
							  		 __global float4 *ctrlPoints,
							  		 __global float4 *l,
							  		 __global float4 *m,
							  		 __global float4 *n,
							  		 __global int4   *topology,
							  		 __global float  *a) {
	
	int i = get_global_id(0);

	float4 u = unitSourceVelocity(i, p, vertices, ctrlPoints, l, m, n,  topology);

	float4 pm = p;
	pm.z = -p.z;

	float4 um = unitSourceVelocity(i, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = n_p.x*(u.x + um.x) + n_p.y*(u.y + um.y) + n_p.z*(u.z - um.z);
}

__kernel void doubletInfluenceNewmann(float4 p,
							  		  float4 n_p,
							  		  __global float4 *vertices,
							  		  __global float4 *ctrlPoints,
							  		  __global float4 *l,
							  		  __global float4 *m,
							  		  __global float4 *n,
							  		  __global int4   *topology,
							  		  __global float  *a) {
	int i = get_global_id(0);

	float4 u = unitDoubletVelocity(i, p, vertices, ctrlPoints, l, m, n, topology);

	float4 pm = p;
	pm.z = -p.z;

	float4 um = unitDoubletVelocity(i, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = n_p.x*(u.x + um.x) + n_p.y*(u.y + um.y) + n_p.z*(u.z - um.z);
}

__kernel void sourceInfluenceDirichlet(float4 p,
							  		   __global float4 *vertices,
							  		   __global float4 *ctrlPoints,
							  		   __global float4 *l,
							  		   __global float4 *m,
							  		   __global float4 *n,
							  		   __global int4   *topology,
							  		   __global float  *a) {
	int i = get_global_id(0);

	float phi = unitSourcePotential(i, p, vertices, ctrlPoints, l, m, n, topology);

	float4 pm = p;
	pm.z = -p.z;

	float phim = unitSourcePotential(i, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = phi + phim;	
}

__kernel void doubletInfluenceDirichlet(float4 p,
							   		  	__global float4 *vertices,
							   		 	__global float4 *ctrlPoints,
							   		 	__global float4 *l,
							   		 	__global float4 *m,
							   		  	__global float4 *n,
							   		  	__global int4   *topology,
							   		  	__global float  *a){
	int i = get_global_id(0);

	float phi = unitDoubletPotential(i, p, vertices, ctrlPoints, l, m, n, topology);

	float4 pm = p;
	pm.z = -p.z;

	float phim = unitDoubletPotential(i, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = phi + phim;
}

__kernel void stripInfluenceDirichlet(float4 p,
									  __global float4 *vertices,
									  __global float4 *ctrlPoints,
									  __global float4 *l,
									  __global float4 *m,
									  __global float4 *n,
									  __global int4   *topology,
									  __global int    *polyInStrip,
									  __global float  *a) {
	int i = get_global_id(0); 

	int iPoly = polyInStrip[i]; // index of current wake polygon

	float phi = unitDoubletPotential(iPoly, p, vertices, ctrlPoints, l, m, n, topology);

	float4 pm = p;
	pm.z = -p.z;

	float phim = unitDoubletPotential(iPoly, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = phi + phim;

}

__kernel void stripInfluenceNewmann(float4 p,
									float4 n_p,
									__global float4 *vertices,
									__global float4 *ctrlPoints,
									__global float4 *l,
									__global float4 *m,
									__global float4 *n,
									__global int4   *topology,
									__global int    *polyInStrip,
									__global float  *a) {
	int i = get_global_id(0); 

	int iPoly = polyInStrip[i]; // index of current wake polygon

	float4 u = unitDoubletVelocity(iPoly, p, vertices, ctrlPoints, l, m, n, topology);

	float4 pm = p;
	pm.z = -p.z;

	float4 um = unitDoubletVelocity(iPoly, pm, vertices, ctrlPoints, l, m, n, topology);

	a[i] = n_p.x*(u.x + um.x) + n_p.y*(u.y + um.y) + n_p.z*(u.z - um.z);

}

__kernel void velocitySource(float4 p,
							 __global float4 *vertices,
							 __global float4 *ctrlPoints,
							 __global float4 *l,
							 __global float4 *m,
							 __global float4 *n,
							 __global int4   *topology,
							 __global float  *sigma,
							 __global float4 *u) {
	int i = get_global_id(0);

	float4 uOut = unitSourceVelocity(i, p, vertices, ctrlPoints, l, m, n,  topology);

	float4 pm = p;
	pm.z = -p.z;

	float4 uOut_m = unitSourceVelocity(i, pm, vertices, ctrlPoints, l, m, n,  topology);

	u[i].x = sigma[i]*(uOut.x + uOut_m.x);
	u[i].y = sigma[i]*(uOut.y + uOut_m.y);
	u[i].z = sigma[i]*(uOut.z - uOut_m.z);
}

__kernel void velocityDoublet(float4 p,
							  __global float4 *vertices,
							  __global float4 *ctrlPoints,
							  __global float4 *l,
							  __global float4 *m,
							  __global float4 *n,
							  __global int4   *topology,
							  __global float  *gamma,
							  __global float4 *u) {
	int i = get_global_id(0);

	float4 uOut = unitDoubletVelocity(i, p, vertices, ctrlPoints, l, m, n,  topology);

	float4 pm = p;
	pm.z = -p.z;

	float4 uOut_m = unitDoubletVelocity(i, pm, vertices, ctrlPoints, l, m, n,  topology);

	u[i].x = gamma[i]*(uOut.x + uOut_m.x);
	u[i].y = gamma[i]*(uOut.y + uOut_m.y);
	u[i].z = gamma[i]*(uOut.z - uOut_m.z);

}
