close all
clear all

% Script that calculates the speed of series 60 for different wind
% idrections and widn speeds, using wing sails as the only propulsion

load('ShipForcesSailsOnlySeries60.mat')
load('FittedValues.mat')
load('wingDataSeries60.mat')

nrIterations = 60;
error = 0.001;

% Environment data
rho_m = 997.561; %kg/m^3
rho_s = 1025.9;  %kg/m^3
nu_m = 1.19E-06; %m^2/s
nu_s = 8.91E-07; %m^2/s
rho_air = 1.225; %kg/m^3

% Sail data
nrSails = 8;
heightSails = 40; %m
chord = 8; %m
areaSail = heightSails*chord; %m^2
totalAreaSail = nrSails*areaSail; %m^2

sailFactor = 0.5*rho_air*totalAreaSail;

%% Wind Speed 7
U_wind = 7;

windAngles2 = 25:1:150;
nrWind2 = length(windAngles2);

U_noYaw = zeros(1, nrWind2);
U_Yaw = zeros(1, nrWind2);
theta = zeros(1, nrWind2);

[shipAngles2D, U_s2D] = meshgrid(0:2.5:10, U_s) ;
shipAngles = 0:2.5:10;

for i = 1:nrWind2
    % No yaw effects on resistance
    U_ship = 0;
    
    u_wind = U_wind*cosd(windAngles2(i));
    
    v_apparent = U_wind*sind(windAngles2(i));
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Thrust = Cx_current*sailFactor*U_apparent;
        
        U_ship = Ufit(0, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_noYaw(i) = U_ship;
    
    % Include yaw effects on resistance
    U_ship = min(U_s);
    yaw_ship = 0;
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Cy_current = interp1(windAngles, CyMax, theta, 'spline');
        
        Side = Cy_current*sailFactor*U_apparent;
        Thrust = Cx_current*sailFactor*U_apparent;
        
        yaw_ship = angFit(U_ship, abs(Side));
        
        U_ship = Ufit(yaw_ship, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_Yaw(i) = U_ship;
    theta(i) = yaw_ship;
end

plot(windAngles2, U_noYaw/0.51444444, '--', 'color', 'blue')
hold on
plot(windAngles2, U_Yaw/0.51444444, 'color', 'blue')


%% Wind speed 10
U_wind = 10;

windAngles2 = 25:1:150;
nrWind2 = length(windAngles2);

U_noYaw = zeros(1, nrWind2);
U_Yaw = zeros(1, nrWind2);
theta = zeros(1, nrWind2);

[shipAngles2D, U_s2D] = meshgrid(0:2.5:10, U_s) ;
shipAngles = 0:2.5:10;

for i = 1:nrWind2
    % No yaw effects on resistance
    U_ship = 0;
    
    u_wind = U_wind*cosd(windAngles2(i));
    
    v_apparent = U_wind*sind(windAngles2(i));
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Thrust = Cx_current*sailFactor*U_apparent;
        
        U_ship = Ufit(0, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_noYaw(i) = U_ship;
    
    % Include yaw effects on resistance
    U_ship = min(U_s);
    yaw_ship = 0;
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Cy_current = interp1(windAngles, CyMax, theta, 'spline');
        
        Side = Cy_current*sailFactor*U_apparent;
        Thrust = Cx_current*sailFactor*U_apparent;
        
        yaw_ship = angFit(U_ship, abs(Side));
        
        U_ship = Ufit(yaw_ship, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_Yaw(i) = U_ship;
    theta(i) = yaw_ship;
end

plot(windAngles2, U_noYaw/0.51444444, '--', 'color', [0, 0.6, 0])
plot(windAngles2, U_Yaw/0.51444444, 'color', [0, 0.6, 0])

%% Wind speed 5
U_wind = 5;

windAngles2 = 25:1:150;
nrWind2 = length(windAngles2);

U_noYaw = zeros(1, nrWind2);
U_Yaw = zeros(1, nrWind2);
theta = zeros(1, nrWind2);

[shipAngles2D, U_s2D] = meshgrid(0:2.5:10, U_s) ;
shipAngles = 0:2.5:10;

for i = 1:nrWind2
    % No yaw effects on resistance
    U_ship = 0;
    
    u_wind = U_wind*cosd(windAngles2(i));
    
    v_apparent = U_wind*sind(windAngles2(i));
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Thrust = Cx_current*sailFactor*U_apparent;
        
        U_ship = Ufit(0, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_noYaw(i) = U_ship;
    
    % Include yaw effects on resistance
    U_ship = min(U_s);
    yaw_ship = 0;
    
    lookingForU = 1;
    
    n = 1;
    while lookingForU
        U_test = U_ship;
        
        u_apparent = u_wind + U_ship;
        
        U_apparent = u_apparent^2 + v_apparent^2;
        
        theta = atan2(v_apparent, u_apparent)*180/pi; %deg
        
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Cy_current = interp1(windAngles, CyMax, theta, 'spline');
        
        Side = Cy_current*sailFactor*U_apparent;
        Thrust = Cx_current*sailFactor*U_apparent;
        
        yaw_ship = angFit(U_ship, abs(Side));
        
        U_ship = Ufit(yaw_ship, Thrust);
        
        if abs(U_test - U_ship)/U_ship < error
            lookingForU = 0;
        elseif n > nrIterations
            lookingForU = 0;
            disp('convergence not reached')
        end
        
        n = n+1;
    end
    
    U_Yaw(i) = U_ship;
    theta(i) = yaw_ship;
end

plot(windAngles2, U_noYaw/0.51444444, '--', 'color', 'red')
plot(windAngles2, U_Yaw/0.51444444, 'color', 'red')


legend( 'Wind speed = 7 m/s, without yaw', 'Wind speed = 7 m/s', 'Wind speed = 10 m/s, without yaw', 'Wind speed = 10 m/s', 'Wind speed = 5 m/s, without yaw', 'Wind speed = 5 m/s',  4)

axis([0, 180, 0, 17])
xlabel('True wind direction [deg]')
ylabel('Ship speed [knots]')
