clear all
close all

load('wingDataChemicalTanker.mat')
load('ShipForcesChemicalTanker.mat')

%% Plot Settings
color = zeros(3);
color(1, :) = [0, 0, 1.0];
color(2, :) = [0, 0.6, 0];
color(3, :) = [1.0, 0, 0];

figure(1);
hold on
figure(2);
hold on
figure(3);
hold on

%% Environment data
rho_m = 997.561; %kg/m^3
rho_s = 1025.9;  %kg/m^3
nu_m = 1.19E-06; %m^2/s
nu_s = 8.91E-07; %m^2/s
rho_air = 1.225; %kg/m^3

Uwind = 7; %m/s

%% Sail data
nrSails = 8;
heightSails = 40; %m
chord = 15; %m
areaSail = heightSails*chord; %m^2
totalAreaSail = nrSails*areaSail; %m^2

sailFactor = 0.5*rho_air*totalAreaSail;

%% Ship data
shipAngles = 0:2:8;

%% Stability data
D = 10;
B = 29.5;
nabla = 38147;

KB = 5.215;
Ixx = 266172;
BM = Ixx/nabla;

stabSin = sind(5);


%% Different wind directions
windAngles2 = 15:1:150;
nrWind2 = length(windAngles2);
for k = 1:3
    % Initialize variables denpendent on wind direction
    KG = zeros(1, nrWind2);

    Thrust = zeros(1, nrWind2);
    SideForce_wing = zeros(1, nrWind2);
    yawAngle = zeros(1, nrWind2);
    ResistanceWind = zeros(1, nrWind2);
    AddedResistance = zeros(1, nrWind2);
    RealThrust = zeros(1, nrWind2); 

    % Go through each wind direction
    for i = 1:nrWind2
        % Calculate apparent wind
        u_apparent = Uwind*cosd(windAngles2(i)) + U_s(k);
        v_apparent = Uwind*sind(windAngles2(i)); 
    
        theta = atan2(v_apparent,u_apparent)*180/pi;
    
        Uapparent = sqrt(u_apparent^2 + v_apparent^2); 
    
        % Thrust and side force coefficients
        Cx_current = interp1(windAngles, CxMax, theta, 'spline');
        Cy_current = interp1(windAngles, CyMax, theta, 'spline');
    
        Thrust(i) = Cx_current*sailFactor*Uapparent^2;
        SideForce_wing(i) = Cy_current*sailFactor*Uapparent^2; 
    
        yawAngle(i) = interp1(SideForce(k, :), shipAngles, abs(SideForce_wing(i)), 'spline');
    
        ResistanceWind(i) = interp1(shipAngles, Resistance(k, :), yawAngle(i), 'spline');
    
        AddedResistance(i) = ResistanceWind(i) - Resistance(k, 1);
    
        RealThrust(i) = Thrust(i) - AddedResistance(i);
        
        KG(i) = (9.81*rho_s*nabla*(KB + BM)*stabSin - abs(SideForce_wing(i))*(heightSails/2 + D))/(9.81*rho_s*nabla*stabSin - abs(SideForce_wing(i)));
    end
    
    figure(1);
    plot(windAngles2, Thrust/Resistance(k, 1), '--','color', color(k, :))
    plot(windAngles2, RealThrust/Resistance(k, 1), 'color', color(k, :))
    
    figure(2);
    plot(windAngles2, yawAngle, 'color', color(k, :))
    
    figure(3);
    plot(windAngles2, KG, 'color', color(k, :))
    
end

% Thrust plotting
figure(1);
legend('U_s = 10 knots, without yaw angle', 'U_s = 10 knots, with yaw angle', 'U_s = 12.5 knots, without yaw angle', 'U_s = 12.5 knots, with yaw angle', 'U_s = 15 knots, without yaw angle', 'U_s = 15 knots, with yaw angle')
xlabel('True wind direction [deg]')
ylabel('(Thrust - added resistance)/Reference resistance')
axis([0, 180, 0, 1])

% Yaw angle plotting
figure(2);
legend('U_s = 10 knots', 'U_s = 12.5 knots', 'U_s = 15 knots', 1)
xlabel('True wind direction [deg]')
ylabel('Required Yaw angle of ship [deg]')
axis([0, 180, 0, 3])

% KG plotting
figure(3)
ylabel('Maximum value of KG [m]') 
legend('U_s = 10 knots', 'U_s = 12.5 knots', 'U_s = 15 knots', 2)
xlabel('True wind direction [deg]')
axis([0, 180, 12, 12.2])
