close all
clear all

load('NACA0014_Data.mat')
% Script that calculates thrust and side force coefficients for different
% wind directions and angles of attack

% Wind directions are Apparent wind directions

figure(1)
hold on

figure(2)
hold on

%% Series 60
windAngles = 5:1:150;
nrWind = length(windAngles);

wingAngles = 0:0.1:24;
nrWing = length(wingAngles);

Cx = zeros(nrWind, nrWing);
Cy = zeros(nrWind, nrWing);

Cx_elliptic = zeros(nrWind, nrWing);
Cy_elliptic = zeros(nrWind, nrWing);

Cx_rectangular = zeros(nrWind, nrWing);
Cy_rectangular = zeros(nrWind, nrWing);

% Wing factors
liftRec = 0.776;
interAng = [30, 60, 90, 120, 150];
liftInter = [0.6862, 0.7429, 0.7168, 0.6818, 0.6232];
dragInter = [2.0098, 1.7330, 1.4302, 1.3254, 1.3814];
Asp = 5*2;
delta = 0.076;

parfor i = 1:nrWind
    liftInterCurrent = interp1(interAng, liftInter, windAngles(i), 'spline');
    dragInterCurrent = interp1(interAng, dragInter, windAngles(i), 'spline');
    
    s = sind(windAngles(i));
    c = cosd(windAngles(i));
    
    for j = 1:nrWing
        CL2D_current = interp1(angleFoil, CL2D, wingAngles(j), 'spline');
        CDv_Current = interp1(angleFoil, CDv, wingAngles(j), 'spline');
        
        CL_elliptic = CL2D_current/(1+2/Asp);
        CL_rectangular = CL2D_current*liftRec;
        CL = CL2D_current*liftRec*liftInterCurrent;
        
        CDi_elliptic = CL_elliptic^2/(pi*Asp);
        CDi_rectangular = (1+delta)*CL_rectangular^2/(pi*Asp);
        CDi = dragInterCurrent*(1+delta)*CL^2/(pi*Asp);
        
        CD_elliptic = CDi_elliptic + CDv_Current;
        CD_rectangular = CDi_rectangular + CDv_Current;
        CD = CDi + CDv_Current;
        
        Cx(i, j) =  CL*s - CD*c;
        Cy(i, j) = -CL*c - CD*s;
        
        Cx_elliptic(i, j) =  CL_elliptic*s - CD_elliptic*c;
        Cy_elliptic(i, j) = -CL_elliptic*c - CD_elliptic*s;
        
        Cx_rectangular(i, j) =  CL_rectangular*s - CD_rectangular*c;
        Cy_rectangular(i, j) = -CL_rectangular*c - CD_rectangular*s;
    end
end

[CxMax, index] = max(Cx, [], 2);
[CxMax_rectangular, index_rectangular] = max(Cx_rectangular, [], 2);
[CxMax_elliptic, index_elliptic] = max(Cx_elliptic, [], 2);

CyMax = zeros(1, nrWind);
CyMax_rectangular = zeros(1, nrWind);
CyMax_elliptic = zeros(1, nrWind);

testAngle = zeros(1, nrWind);

parfor i = 1:nrWind
    CyMax(i) = Cy(i, index(i));
    CyMax_rectangular(i) = Cy_rectangular(i, index_rectangular(i));
    CyMax_elliptic(i) = Cy_elliptic(i, index_elliptic(i));
    
    testAngle(i) = wingAngles(index(i))
end

figure(1)
plot(windAngles, CxMax, 'color', 'blue')
plot(windAngles, CxMax_rectangular, 'color', [0, 0.6, 0])
plot(windAngles, CxMax_elliptic, 'color', 'red')

xlabel('Apparent wind direction [deg]')
ylabel('thrust coefficient, C_x')
legend('8 rectangular wings', 'Single rectangular wing', 'Single elliptic wing', 4)
axis([0, 180, 0, 1.5])
title('Series 60 Wings')

figure(2)
plot(windAngles, CyMax, 'color', 'blue')
plot(windAngles, CyMax_rectangular, 'color', [0, 0.6, 0])
plot(windAngles, CyMax_elliptic, 'color', 'red')

axis([0, 180, -1.5, 1.5])

xlabel('Apparent wind direction [deg]')
ylabel('Side force coefficient, C_y')

legend('8 rectangular wings', 'Single rectangular wing', 'Single elliptic wing', 2)
title('Series 60 Wings')
